/* ===========================================================================
                      TOPOLOGICAL SORT
This program uses the module for lists in the file `list_module.cpp'.
We want to compute a total order of N values which is consistent with a given
partial order. The partial order among the N values 0,...,N-1, is stored as
two arrays: for each value, that, is, for i=0 to N-1,
- countofsmaller[i] is the number of values smaller than i, and
- listoflarger[i] is the list of values larger than i.
The partial order is given in a file named `partialorder_in', where there is a
sequence of pairs. The pair `n m' tells us that n <= m in the given partial
order <=. In the pair `n m' n is said to be the smallvalue and m is said to be
the largevalue.
REMARKS. In every pair `n m' in the file `partialorder_in':
(1) n should be different from m, because reflexivity is assumed, and
(2) if we have the pair `n m', then we should not have the pair `m n'.
The zerolist is the list of values which do not have smaller values waiting to
be inserted in the total order.
------------------------------------------------------------------------------
Consider N=9 values, and the file `partialorder_in' whose content is:
3 4 3 8 0 2 0 7 2 7 7 5 2 6 2 5 3 2     meaning that 3<=4, 3<=8, etc.
We have the following two arrays:
  0   1   2   3   4   5   6   7   8   :smallvalue (index of the array)
[ 0   0   2   0   1   2   1   2   1 ] :countofsmaller

[ 8  nil  6   3  nil nil nil  6  nil] :listoflarger
  |       |   |               |
  v       v   v               v
  3       7   9              nil
  |       |   |
  v       v   v
 nil      8   5
          |   |
          v   v
         nil nil
=========================================================================== */
#include <fstream.h>       // File needed for using files.
typedef int Values;        // Values are assumed to be integers.
#include "list_module.cpp" // We have lists of int.
const int N=9;             // The values are from 0 to 8.
fstream input_po;          // We use a file for describing the partial order.
int countofsmaller [N];
list listoflarger  [N];
list zerolist=NULL;
//----------------------------------------------------------------------------
void printar(int A[], int size)
{for (int i=0; i<size; i=i+1) {cout << A[i] << " ";}}
//----------------------------------------------------------------------------
void printpo(int countofsmaller [], list listoflarger  [], list zerolist)
{cout << "\ncountofsmaller [0.."<< N-1 <<"]:\n   ";
printar(countofsmaller,N);
cout << "\n\nlistoflarger [0.."<< N-1 <<"]:\n   ";
for (int i=0; i<N; i=i+1)
   {cout << i << ": "; printlist(listoflarger[i]);
    cout << "\n   ";};
cout << "\nzerolist:\n   "; printlist(zerolist);
cout << "\n\n";}
//----------------------------------------------------------------------------
void readpo(int countofsmaller [], list listoflarger  [], list &zerolist)
{int smallvalue=0; int largevalue=0;
for (int i=0; i<N-1; i=i+1) {listoflarger[i]=NULL; countofsmaller[i]=0;};
input_po >> smallvalue;   // <-- eof()=1 only after trying
while (!input_po.eof())   //   to read a non existing item
     {input_po >> largevalue;
      listoflarger[smallvalue] = cons(largevalue,listoflarger[smallvalue]);
      countofsmaller[largevalue]++;
      input_po >> smallvalue; // <-- trying to read the first node
      };                      // of a new <smallvalue, largevalue>
for (int i=0; i<N-1; i=i+1)
{if (countofsmaller[i]==0) {zerolist=cons(i,zerolist);};};
}
//----------------------------------------------------------------------------
void main()
{cout <<"Please, prepare a file called";
cout <<" `partialorder_in',\n";
cout <<"as indicated in the initial comments of the program.\n";
input_po.open("partialorder_in", ios::in);
readpo(countofsmaller,listoflarger,zerolist);
printpo(countofsmaller,listoflarger,zerolist);
cout <<"Consistent total order:\n   ";
while (zerolist!=NULL)
  {int v=head(zerolist); cout << v <<" "; zerolist=tail(zerolist);
         while (listoflarger[v]!=NULL)
                  {int h=head(listoflarger[v]); countofsmaller[h]--;
                   if (countofsmaller[h]==0) {zerolist=cons(h,zerolist);};
                   listoflarger[v]=tail(listoflarger[v]);};
  };
cout <<"\n";
}
/* input:   output:
------------------------------------------------------------------------------
    Please, prepare a file called `partialorder_in',
    as indicated in the initial comments of the program.

    countofsmaller [0..8]:
       0 0 2 0 1 2 1 2 1

    listoflarger [0..8]:
       0: 7->2->nil
       1: nil
       2: 5->6->7->nil
       3: 2->8->4->nil
       4: nil
       5: nil
       6: nil
       7: 5->nil
       8: nil

    zerolist:
       3->1->0->nil

    Consistent total order:
       3 4 8 1 0 2 7 5 6
--------------------------------------------------------------------------- */